@@ -36,7 +36,7 @@
 #include "ScrollTypes.h"
 #include "UserScriptTypes.h"
 #include <memory>
-#include <wtf/RefCounted.h>
+#include <wtf/ThreadSafeRefCounted.h>
 
 #if PLATFORM(IOS)
 #include "ViewportArguments.h"
@@ -113,7 +113,7 @@ namespace WebCore {
     };
     typedef unsigned LayerTreeFlags;
 
-    class Frame : public RefCounted<Frame> {
+    class Frame : public ThreadSafeRefCounted<Frame> {
     public:
         WEBCORE_EXPORT static Ref<Frame> create(Page*, HTMLFrameOwnerElement*, FrameLoaderClient*);
 
@@ -223,7 +223,7 @@ namespace WebCore {
 
         WEBCORE_EXPORT String displayStringModifiedByEncoding(const String&) const;
 
-        WEBCORE_EXPORT VisiblePosition visiblePositionForPoint(const IntPoint& framePoint);
+        WEBCORE_EXPORT VisiblePosition visiblePositionForPoint(const IntPoint& framePoint) const;
         Document* documentAtPoint(const IntPoint& windowPoint);
         WEBCORE_EXPORT RefPtr<Range> rangeForPoint(const IntPoint& framePoint);
 
@@ -379,7 +379,7 @@ namespace WebCore {
 
     inline void Frame::detachFromPage()
     {
-        m_page = 0;
+        m_page = nullptr;
     }
 
     inline EventHandler& Frame::eventHandler() const
