@@ -26,6 +26,7 @@
 #include "config.h"
 #include "RenderTreeAsText.h"
 
+#include "ClipRect.h"
 #include "Document.h"
 #include "FlowThreadController.h"
 #include "Frame.h"
@@ -483,7 +484,7 @@ static void writeTextRun(TextStream& ts, const RenderText& o, const InlineTextBo
     ts << "\n";
 }
 
-static void writeSimpleLine(TextStream& ts, const RenderText& o, const LayoutRect& rect, StringView text)
+static void writeSimpleLine(TextStream& ts, const RenderText& o, const FloatRect& rect, StringView text)
 {
     int x = rect.x();
     int y = rect.y();
@@ -548,7 +549,7 @@ void write(TextStream& ts, const RenderObject& o, int indent, RenderAsTextBehavi
                 writeSimpleLine(ts, text, run.rect(), run.text());
             }
         } else {
-            for (auto box = text.firstTextBox(); box; box = box->nextTextBox()) {
+            for (auto* box = text.firstTextBox(); box; box = box->nextTextBox()) {
                 writeIndent(ts, indent + 1);
                 writeTextRun(ts, text, *box);
             }
@@ -666,9 +667,9 @@ static void writeRenderRegionList(const RenderRegionList& flowThreadRegionList,
             RenderElement* renderElementForRegion = isRenderNamedFlowFragment ? renderRegion->parent() : renderRegion;
             if (renderElementForRegion->isPseudoElement()) {
                 if (renderElementForRegion->element()->isBeforePseudoElement())
-                    tagName.append("::before");
+                    tagName.appendLiteral("::before");
                 else if (renderElementForRegion->element()->isAfterPseudoElement())
-                    tagName.append("::after");
+                    tagName.appendLiteral("::after");
             }
 
             ts << " {" << tagName.toString() << "}";
@@ -809,7 +810,7 @@ static String nodePosition(Node* node)
 {
     StringBuilder result;
 
-    Element* body = node->document().body();
+    auto* body = node->document().bodyOrFrameset();
     Node* parent;
     for (Node* n = node; n; n = parent) {
         parent = n->parentOrShadowHostNode();
